﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ImageProcessor;

namespace Image_Processor
{
	class CustomImageProcessor : IDisposable
	{
		private bool disposed = false;
		private string filePath;
		private byte[] photoBytes;
		private MemoryStream inStream;
		private ImageFactory imageFactory;

		public CustomImageProcessor(string filePath)
		{
			this.filePath = filePath;
			photoBytes = File.ReadAllBytes(filePath);
			inStream = new MemoryStream(photoBytes);
			imageFactory = new ImageFactory(preserveExifData: true);
		}

		public void Dispose()
		{
			Dispose(true);
			GC.SuppressFinalize(this);
		}

		protected virtual void Dispose(bool disposing)
		{
			if (!disposed)
			{
				if (disposing)
				{
					inStream.Dispose();
					imageFactory.Dispose();
				}
				disposed = true;
			}
		}

		public string GetFormat()
		{
			return filePath.Substring(filePath.LastIndexOf('.'));
		}

		public string GetDir()
		{
			return Path.GetDirectoryName(filePath) + '\\';
		}

		public string GetFileName()
		{
			int pathLen = GetDir().Length;
			int nameLen = filePath.IndexOf('.') - pathLen;
			return filePath.Substring(pathLen, nameLen);
		}

		private void ApplyProcess(ref ImageFactory imageFactory, ProcessMethod process)
		{
			switch (process.name)
			{
				case ProcessMethod.Name.Brightness:
					imageFactory.Brightness(process.value);
					break;
				case ProcessMethod.Name.Alpha:
					imageFactory.Alpha(process.value);
					break;
				case ProcessMethod.Name.Saturation:
					imageFactory.Saturation(process.value);
					break;
				case ProcessMethod.Name.Contrast:
					imageFactory.Contrast(process.value);
					break;
				case ProcessMethod.Name.EdgeDetection:
					bool greyscale = process.value <= 10;
					imageFactory.DetectEdges(process.GetEdgeDetectionFilter(), greyscale);
					break;
				case ProcessMethod.Name.Filter:
					imageFactory.Filter(process.GetMatrixFilter());
					break;
				case ProcessMethod.Name.Tint:
					imageFactory.Tint(process.color);
					break;
				case ProcessMethod.Name.Hue:
					bool rotate = process.value < 1000;
					int value = rotate ? process.value : process.value - 1000;
					imageFactory.Hue(value, rotate);
					break;
				case ProcessMethod.Name.Blur:
					imageFactory.GaussianBlur(process.value);
					break;
				case ProcessMethod.Name.Sharpen:
					imageFactory.GaussianSharpen(process.value);
					break;
				case ProcessMethod.Name.ReplaceColor:
					imageFactory.ReplaceColor(process.color, process.secondaryColor, process.value);
					break;
				case ProcessMethod.Name.Vignette:
					imageFactory.Vignette(process.color);
					break;
			}
		}

		public void ProcessImage(ProcessSequence processes)
		{
			imageFactory.Load(inStream);
			foreach (ProcessMethod process in processes.GetProcesses())
			{
				ApplyProcess(ref imageFactory, process);
			}
		}

		public Image GetImage()
		{
			return imageFactory.Image;
		}

		async public Task SaveImage(string fileName, string directory="")
		{
			if (directory.Length == 0)
			{
				directory = GetDir();
			}
			string filePath = directory + GetFileName() + fileName + GetFormat();
			await Task.Run(() => imageFactory.Save(filePath));
		}

		~CustomImageProcessor()
		{
			Dispose(false);
		}
	}
}
